@@ -23,6 +23,7 @@ import os.path
 
 from accerciser.plugin import ViewportPlugin
 from accerciser.icons import getIcon
+from accerciser import node
 from accerciser.i18n import _, N_, DOMAIN
 from xml.dom import minidom
 
@@ -1460,8 +1461,26 @@ class _SectionText(_InterfaceSection):
     @param param_spec: Some gobject crud
     @type param_spec: object
     '''
+
     self.offset_spin.set_value(text_buffer.get_property('cursor-position'))
 
+    s = text_buffer.get_selection_bounds()
+    if s != ():
+      # Highlight selected text
+      try:
+        start,end = s
+        startOffset = start.get_offset()
+        endOffset = end.get_offset()
+        text = self.node.acc.queryText()
+        (x, y, width, height) = text.getRangeExtents(startOffset, endOffset, pyatspi.DESKTOP_COORDS)
+        ah = node._HighLight(x, y, width, height,
+                             node.FILL_COLOR, node.FILL_ALPHA,
+                             node.BORDER_COLOR, node.BORDER_ALPHA,
+                             2.0, 0)
+        ah.highlight(node.HL_DURATION)
+      except:
+        pass
+
   def _accEventText(self, event):
     '''
     Callback for accessible text changes. Updates the text buffer accordingly.
